# Wendelin.core | pytest config
# Copyright (C) 2020  Nexedi SA and Contributors.
#                     Kirill Smelkov <kirr@nexedi.com>
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.

from __future__ import print_function, absolute_import

from golang import func, defer
from functools import partial

# Before pytest exits, teardown WCFS(s) that we automatically spawned during
# test runs in bigfile/bigarray/...
#
# If we do not do this, spawned wcfs servers are left running _and_ connected
# by stdout to nxdtest input - which makes nxdtest to wait for them to exit.
@func
def pytest_unconfigure(config):
    from wendelin import wcfs
    print("\n\nspawned WCFS:")
    print(wcfs._wcstarted)
    for wc in wcfs._wcstarted:
        if wc._proc.poll() is not None:
            continue # this wcfs server already exited

        # FIXME currently has to kill wcfs.go, because swapned pinner threads are running
        #       -> stop pinner first
        # NOTE: defer instead of direct call - to call all wc.close if there was multiple wc spawned
        defer(partial(_wcclose, wc))


def _wcclose(wc):
    from wendelin.wcfs.wcfs_test import tWCFS
    print("(unmount/stop wcfs pid%d @ %s)" % (wc._proc.pid, wc.mountpoint))
    twc = tWCFS(wc=wc)
    twc.close()
